/***************************************************************************
 *   Copyright (C) 2008 by Michał Pilch  <mpilhoo@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "graphicsrowcollayout.h"
#include <KDebug>

GraphicsRowColLayout::GraphicsRowColLayout(QGraphicsLayoutItem *parent)
{
kDebug() << "";
	items.clear();
}

GraphicsRowColLayout::~GraphicsRowColLayout() {}

void GraphicsRowColLayout::setOrientation(Qt::Orientation orientation){
kDebug() << ((orientation == Qt::Vertical) ? "Vertical":"Horizontal");
	m_orientation=orientation;
}

Qt::Orientation GraphicsRowColLayout::orientation() const {
kDebug() << ((m_orientation == Qt::Vertical) ? "Vertical":"Horizontal");
	return m_orientation;
}

void GraphicsRowColLayout::insertItem(int index, QGraphicsLayoutItem *item){
kDebug() << "";
	items.append(item);
	if(index>-1)
		moveFromTo(items.count()-1, index);
	updateItemsGeometry();
}

void GraphicsRowColLayout::removeItem(QGraphicsLayoutItem *item){
kDebug() << "";
	for(int i=0;i<items.count();i++){
		if(item==items[i]){
			removeAt(i);
			break;
		}
	}
	updateItemsGeometry();
}

void GraphicsRowColLayout::removeAt(int index){
kDebug() << "";
	moveFromTo(index,items.count()-1);
	items.remove(items.count()-1);
	updateItemsGeometry();
}

void GraphicsRowColLayout::setGeometry(const QRectF &rect){
kDebug() << "";
	QGraphicsLayoutItem::setGeometry(rect);
	updateItemsGeometry();
}

QSizeF GraphicsRowColLayout::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const {
kDebug() << "";
	// TODO I have no idea how this method should work :(
// 	QGraphicsLayoutItem::sizeHint(which, constraint);
}

int GraphicsRowColLayout::count() const {
kDebug() << "";
	return items.count();
}

QGraphicsLayoutItem* GraphicsRowColLayout::itemAt(int index) const {
kDebug() << "";
	return items[index];
}

void GraphicsRowColLayout::setRowsCols(int r, int c){
kDebug() << "";
	if(m_minRows!=r){
		m_minRows=r;
	}
	if(m_minColumns!=r){
		m_minColumns=c;
	}
	updateItemsGeometry();
}

int GraphicsRowColLayout::rows(){
kDebug() << "";
	return m_rows;
}
int GraphicsRowColLayout::columns(){
kDebug() << "";
	return m_columns;
}

int GraphicsRowColLayout::minRows(){
kDebug() << "";
	return m_minRows;
}
int GraphicsRowColLayout::minColumns(){
kDebug() << "";
	return m_minColumns;
}

void GraphicsRowColLayout::updateItemsGeometry(){
kDebug() << "";
	if(orientation()==Qt::Horizontal){
		m_rows=m_minRows;
		m_columns=(m_minColumns > (items.count()+m_rows-1)/m_rows) ? m_minColumns : (items.count()+m_rows-1)/m_rows;
		
		m_VItemSize=geometry().width()/(qreal)m_columns;
		m_HItemSize=geometry().height()/(qreal)m_rows;

		for(int i=0;i<items.count();i++)
			items[i]->setGeometry(QRectF((qreal)(i/m_rows)*m_VItemSize, (qreal)(i%m_rows)*m_HItemSize, m_VItemSize, m_HItemSize));
		
	}else{
		m_columns=m_minColumns;
		m_rows=(m_minRows > (items.count()+m_columns-1)/m_columns) ? m_minRows:(items.count()+m_columns-1)/m_columns;
		
		m_VItemSize=geometry().width()/(qreal)m_columns;
		m_HItemSize=geometry().height()/(qreal)m_rows;
		
		for(int i=0;i<items.count();i++)
			items[i]->setGeometry(QRectF((qreal)(i%m_columns)*m_VItemSize, (qreal)(i/m_columns)*m_HItemSize, m_VItemSize, m_HItemSize));
	}

}
