/***************************************************************************
 *   Copyright (C) 2008 by Michał Pilch  <mpilhoo@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef GRAPHICSROWCOLLAYOUT_H
#define	GRAPHICSROWCOLLAYOUT_H

#include <QGraphicsLayout>
#include <QVector>

class GraphicsRowColLayout : public QGraphicsLayout {
public:
	GraphicsRowColLayout(QGraphicsLayoutItem *parent = 0);
	~GraphicsRowColLayout();
	
	void setOrientation(Qt::Orientation orientation);
	Qt::Orientation orientation() const;
	
	inline void addItem(QGraphicsLayoutItem *item) { insertItem(-1, item); }
	inline void moveFromTo(int a,int b){
		QGraphicsLayoutItem *h;
		char d=(a>b) ? -1:1;
		int i=a;
		while(i!=b){
			h=items[i+d];
			items[i+d]=items[i];
			items[i]=h;
			i+=d;
		}
	}

	void insertItem(int index, QGraphicsLayoutItem *item);

	void removeItem(QGraphicsLayoutItem *item);
	void removeAt(int index);

	void setSpacing(qreal spacing);
	qreal spacing() const;
	
	void setGeometry(const QRectF &rect);
	QSizeF sizeHint (Qt::SizeHint, const QSizeF& = QSizeF()) const;
	int count() const;
	QGraphicsLayoutItem* itemAt (int) const;
	void setRowsCols(int r, int c);
	int rows();
	int columns();
	int minRows();
	int minColumns();
private:
	int m_rows, m_columns, m_minRows, m_minColumns;
	qreal m_HItemSize, m_VItemSize;
	QVector<QGraphicsLayoutItem*> items;
	void updateItemsGeometry();
	Qt::Orientation m_orientation;
};

#endif
